
% The algorithm below is a slightly modified version of the code provided
% by Smets and Wouters in the AER webpage. The subroutines, mgnldnsty.m,
% mgnldnsty_fcast are coded by Christopher A. Sims, which are available
% on his webpage: http://sims.princeton.edu/yftp/VARtools/matlab/. These
% subroutines are also used by Smets and Wouters(2007). 

% BVAR with explicit train: starting from 66 onwards
 disp('BVAR(4) forecasting and MLE calculation starting from 66 onwards')
% ======================= initialise data %
% ===============================
% Before giving the details of the code I should mention something crucial
% for the procedure. We have two different set of datasets. First set of
% datasets are the ones that are used in Dynare estimation, which the
% details are already given. The second set are the ones that are used in
% BVAR algorithm below. The second set is created by the names df1, df2,
% .. df64, as in the same fashion with the ones used in Dynare estimation.
% Even though the datafiles used in BVAR have the training sample in the
% begining (we made sure of this by running the code appending), these
% files are 8 observations longer than the Dynare datafiles. The reason for
% this is that, the code below cannot generate forecasts for the missing
% observations. So these 8 observations at the end of the file enable us to 
% calculate the 8 quarters ahead forecasts. For exampe, when we run the loop
% k=1, we see that the datafile for the Dynare estimation has a length of
% 165 observations, but the datafile created for BVAR has a length of 173
% observations. These 8 observations at the end are filled with NIPA and Labor forecasts.
% Actually it doesn't really matter what values you give to the
% observations because they are not considered for the estimation and
% forecasts anyway. Putting zeros is as good as putting the actual
% forecasts. The only purpose of those extra observations are for telling
% the code to calculate the forecasts, nothing more. More on this below.

% So we are loading the datasets generated recursively.


load(sprintf('df%d.mat',k));
% load(sprintf('extdf%d.mat',k)); This is for GB 1992 to 1996.
% load(sprintf('bcdf%d.mat',k));  This is for BC 1997 to 2009.
% load(sprintf('extbcdf%d.mat',k)); This is for BC 1992 to 1996.

% ======================= initialise priors % =============================
    % The priors below are basically the Minnesota Priors that Smets and
    % Wouters(2007) use. 

    T0       = 40;
    maxnlags =  5;
    tau      =  10;
    d        =  1;
    lambda   =  5;    
    mu       =  2;
    omega    =  1;
    breaks   = [];
    train    = T0;
    flat     = 0;

mnprior.tight = tau;
mnprior.decay = d;
zdata=dataset;
yyprior = zdata(1:1+40,:);
yykprior = [ zdata(2:2+40,:) ];
yypriornobs = rows(yyprior);

ccprior = inv(yykprior'*yykprior)*yykprior'*yyprior;
errorAprior = yyprior-yykprior*ccprior;
sigprior=(errorAprior)'*errorAprior/(yypriornobs);
sigbvar_hat=sqrt(diag((errorAprior)'*errorAprior/(yypriornobs)))

vprior.sig=sigbvar_hat';     
vprior.w=1          ;         

% ======================= marg log dens % ====:============================
% Below there are two definitions for options_.first_obs_control. First
% one does not use training sample in MLE and forecasting calculations.This
% is consistent with Smets and Wouters(2007). They also calculate forecasts
% without the training sample.On the other hand the one below(the comment line) 
% uses training sample of 40 observations in calculating forecasts. Note
% that the marginal likelihood calculation below depends on the number of
% observations and the first observation of the estimation undertaken by
% Dynare. The loop below is for marginal likelihood calculation. It
% calculates ML for 5 lags and creates the appropriate dataset for the
% calculation. The variables options_.first_obs, options_.presample and 
% options_.nobs are all specified in Dynare. Notice that if we did not append the 
% training sample on top of the datafiles used in Dynare, then both the first observation and 
% the number of observations will be wrong. Thus the datafiles should be comparable. 
% One condition for this algorithm to work properly is to make sure that the datasets used in
% Dynare estimation and BVAR estimation to have same length. Moreover, as
% mentioned above since there are 8 observations more in the df files and
% these observations are filled with forecasts, we do not want them to be
% considered for ML calculations. Thus we continue with the number of
% observations in the Dynare datafile, which does not include 8 periods of
% observation. One final point is that the Bvar is estimated every quarter.


options_.first_obs_control =  options_.first_obs;

%The command below calculates trained BVAR forecasts and estimations. 
%options_.first_obs_control =  options_.first_obs -T0 ;
options_.nobs_control      =  options_.nobs      ;
for lag = 1:maxnlags
    ydata = dataset(options_.first_obs_control+options_.presample-lag:options_.first_obs+options_.nobs-1,:);
    w=mgnldnsty(ydata,lag,ones(size(ydata,1),1),breaks,lambda,mu,mnprior,vprior,train+lag,flat);
    disp(' ')
    fprintf('The marginal log density of the BVAR(%g) model is equal to %10.4f \n',lag,w);
    disp(' ')
end

% The algorithm below is for calculating BVAR(4) forecasts. 

% =============================== start forecast ==========================
 
lag=4;
ydata = dataset(options_.first_obs_control+options_.presample-lag:options_.first_obs+8+options_.nobs-1,:);
% Now notice that we have added 8 to the ydata line. The reason for this is
% to include the 8 observations that we have filled with forecasts, and
% tell the code to calcuate forecasts for those periods too. While saving
% the bvar forecasts, we save the forecasts that correspond to these 8
% observations. For instance assume that, for k=1, we calcuate the bvar
% forecasts. The datafile, datafile1 will have 124 observations for
% calculation (since our first observation is 42(1966 Q1)) and df1 will have
% 132 observations available for calculation. So the bvar forecasts that we
% are interested in are the ones for observations 125 to 132. Thus we save
% those. 
forhor = 8;
forper = 8;
nobs = 7;
% forhor, forper, nobs specify forecast horizon, forecast period and number
% of observable variables respectively. Since our forecast horizon is 8
% quarters we set forhor = 8. Since we are interested these 8 quarter
% forecasts to start from the end of the dataset, we set forecast period to
% be equal to 8. And the number of observable variables are 7, consumption
% growth, investment growth, gdp growth, hours, inflation and interest
% rates. 
%
gend = size(ydata,1);
% ==================================================================================
% Below loop calculates the BVAR(4) forecasts. 
% ==================================================================================
yhatmat=zeros(forhor,nobs,forper);
fcast_errormat=zeros(forhor,nobs,forper);
for i=gend-forper:gend
% Since we have set forecast horizon and forecast period to 8, the
% forecasts will start from the end of the dataset, say from observation
% 124 if we do it for k=1.  
nhat=min(forhor,gend+1-i);
% The function mgnldnsty_fcast is a subroutine written by Sims. It
% calculates the BVAR forecasts. 
[w,yhat,fcast_error]=mgnldnsty_fcast(ydata,lag,ones(size(ydata,1),1),breaks,lambda,mu,mnprior,vprior,train+lag,flat,i,nhat) ;
yhatmat(1:nhat,:,i-gend+forper+1)=yhat;
% yhatmat is the matrix of forecasts. For k=1, it creates 9 forecast
% matricies. The first one is from 124-131. The second matrix is from
% 125-132, which we are interested in. In the main loop, we are going to
% save the yhatmat(:,:,2) component of every matrix generated by the
% forecast algorithm, since it gives the forecasts that are matched with
% the appropriate periods.
end

% =========================================================================



